function demo_adaboost(datafile, weaklearner, max_rules)
% DEMO_ADABOOST Demonstrates the adaboost algorithm. Requires STPRtoolbox.
%
% Description:
%   ...

% Filename     : demo_adaboost.m
% Author       : Petr Posik (posik#labe.felk.cvut.cz, replace # with @ to get email)
% Created      : 18-Feb-2010 17:21:49
% Modified     : $Date: 2011-03-08 12:09:09 +0100 (út, 08 III 2011) $
% Revision     : $Revision: 7904 $
% Developed in : 7.5.0.342 (R2007b)
% $Id: demo_adaboost.m 7904 2011-03-08 11:09:09Z posik $

    % Process arguments
    if nargin < 1 || isempty(datafile), datafile = 'dataCircle'; end
    if nargin < 2 || isempty(weaklearner), weaklearner = 'weaklearner_perc'; end
    if nargin < 3 || isempty(max_rules), max_rules = 30; end

    % Load data
    clc;
    data = load(datafile);

    % Set the options for adaboost and train it
    N = size(data.X,2);
    options.learner = weaklearner;
    options.max_rules = max_rules;
    options.verb = 1;
    [model, D] = adaboost(data,options);

    % Plot the results
    ax = getAxisForData(data);
    hf1 = presentationfigure; axis(ax);
    hf2 = presentationfigure; axis(ax);

    hx1 = []; hx2 = [];
    hb = []; hh = [];
    hc1 = []; hc2 = [];
    for i = 1:length(model.rule),
        tempmodel = model;
        tempmodel.Alpha = tempmodel.Alpha(1:i);
        tempmodel.Z = tempmodel.Z(1:i);
        tempmodel.WeightedErr = tempmodel.WeightedErr(1:i);
        tempmodel.ErrBound = tempmodel.ErrBound(1:i);
        tempmodel.rule = tempmodel.rule(1:i);
        if ~isempty(hx1), arrayfun(@delete, hx1); end
        if ~isempty(hx2), arrayfun(@delete, hx2); end
        if ~isempty(hb), delete(hb); end
        if ~isempty(hh), delete(hh); end
        if ~isempty(hc1), delete(hc1); end
        if ~isempty(hc2), delete(hc2); end

        % Figure 1: draw weighted points and the decision boundary of the
        % last model
        figure(hf1); hold on;
        title(sprintf('Iter %d: Last hypothesis', i));
        hc1 = pareas(hf1, tempmodel.rule{i});
        hx1 = pwpatterns(data, D(:,i)*N);
        hh = pline(model.rule{i}); set(hh,'Linewidth',3,'color','k');
        
        % Figure 2: draw unweighted points and the decision boundary of the
        % whole model
        figure(hf2); hold on;
        title(sprintf('The whole model up to iter %d', i));
        hc2 = pareas(hf2, tempmodel); 
        hx2 = pwpatterns(data, ones(N,1));
        hb = pboundary(tempmodel); set(hb, 'Linewidth',3,'color','k');
        
        pause;
    end
end