function demo_perceptron(datafile)
% DEMO_PERCEPTRON Shows the perceptron algorithm in action.
% 
% Description:
%   ...

% Filename     : demo_perceptron.m
% Author       : Petr Posik (posik#labe.felk.cvut.cz, replace # with @ to get email)
% Created      : 18-Feb-2010 17:21:49
% Modified     : $Date: 2011-03-08 12:09:09 +0100 (út, 08 III 2011) $
% Revision     : $Revision: 7904 $
% Developed in : 7.5.0.342 (R2007b)
% $Id: demo_perceptron.m 7904 2011-03-08 11:09:09Z posik $

    % Process arguments
    if nargin < 1 || isempty(datafile), datafile = 'dataSeparable'; end

    % Load data
    clc;
    data = load(datafile);

    % Prepare figure
    ax = getAxisForData(data);
    hf1 = presentationfigure; axis(ax); hold on;

    % Prepare data (homogenize, invert one class)
    [dim,N] = size(data.X);
    x = [data.X; ones(1,N)];
    x(:,data.y ~= 1) = -x(:,data.y ~= 1);

    w = x(:,ceil(N*rand));
    D = ones(N,1);
    hx = []; ha = []; hl = [];
    for iter = 1:1000,
        % Find worst classified vector
        [minf, imin] = min(w'*x);
        if minf > 0, break; end
        w = w + 0.05*x(:,imin);
        
        % Update the distribution (the diameter of the point markers)
        % Make them a bit smaller
        D = 0.95*D;
        D(imin) = D(imin)+1;
        % Limit them to 0.4-3 interval
        D = min(D, 3);
        D = max(D, 0.4);
        
        % Visualize
        if ~isempty(hx), arrayfun(@delete, hx); end
        if ~isempty(ha), delete(ha); end
        if ~isempty(hl), delete(hl); end
        model.W = w(1:2);
        model.b = w(3);
        model.fun = 'linclass';
        ha = pareas(gcf, model);
        hx = pwpatterns(data, D); %2*D/max(D));%*N/sum(D));
        hl = pline(model); set(hl,'Linewidth',3,'color','k');
        title(sprintf('Iteration %d', iter));
        
        drawnow;
    end
end
